<?php
namespace WCProductsDependencies;

/**
 * Settings Class
 *
 * @class Settings
 * @version 1.1.0
 */
class Settings
{
    /**
     * Define settings model
     *
     * @param string|null $source - get all or specific only
     *
     * @return array
     */
    public static function getModel($source = null)
    {
        static $modelCache;

        if ($source && $modelCache && isset($modelCache[$source])) {
            return apply_filters('wcpd_settings_model_' . $source, $modelCache[$source]);
        } elseif ($modelCache) {
            return apply_filters('wcpd_settings_models', $modelCache);
        }

        $users = get_users(['fields' => ['ID', 'user_nicename', 'user_email']]);
        $usersData = [];

        foreach ($users as $user) {
            $usersData[$user->ID] = "#$user->ID $user->user_nicename ($user->user_email)";
        }

        $global = [
            'check_customer_ordered_products' => [
                'name' => esc_html__('Check customer ordered products', 'woocommerce-products-dependencies'),
                'default' => false,
                'type' => 'checkbox',
                'id' => 'woocommerce_products_dependencies_check_customer_ordered_products'
            ],
            'check_dependencies_on_checkout_only' => [
                'name' => esc_html__('Check dependencies on checkout page only', 'woocommerce-products-dependencies'),
                'default' => false,
                'type' => 'checkbox',
                'id' => 'woocommerce_products_dependencies_check_dependencies_on_checkout_only'
            ],
            'logged_dependency' => [
                'name' => esc_html__('User must be logged to buy', 'woocommerce-products-dependencies'),
                'default' => false,
                'type' => 'checkbox',
                'id' => 'woocommerce_products_dependencies_logged_dependency'
            ],
            'output_dependency_messages_in_advance' => [
                'name' => esc_html__('Output dependencies messages in advance', 'woocommerce-products-dependencies'),
                'default' => true,
                'type' => 'checkbox',
                'id' => 'woocommerce_products_dependencies_output_messages_in_advance'
            ],
            'logged_dependency_message' => [
                'name' => esc_html__('Logged dependencies message', 'woocommerce-products-dependencies'),
                'default' => esc_html__('To add $product$ to your cart you must be logged in', 'woocommerce-products-dependencies'),
                'desc' => esc_html__('Items for using: $product$ - product name','woocommerce-products-dependencies'),
                'type' => 'text',
                'css' => 'width: 100%',
                'id' => 'woocommerce_products_dependencies_message_logged_title'
            ],
            'buy_once_dependency_message' => [
                'name' => esc_html__('Buy once dependency message', 'woocommerce-products-dependencies'),
                'default' => esc_html__('You already bought $product$ before','woocommerce-products-dependencies'),
                'desc' => esc_html__('Items for using: $product$ - product name', 'woocommerce-products-dependencies'),
                'type' => 'text',
                'css' => 'width: 100%',
                'id' => 'woocommerce_products_dependencies_buy_once_message'
            ],
            'product_categories_dependency_message' => [
                'name' => esc_html__('Categories dependencies message', 'woocommerce-products-dependencies'),
                'default' => esc_html__(
                    'Before adding $product$ to your cart need to choose something from category:',
                    'woocommerce-products-dependencies'
                ),
                'desc' => esc_html__('Items for using: $product$ - product name', 'woocommerce-products-dependencies'),
                'type' => 'text',
                'css' => 'width: 100%',
                'id' => 'woocommerce_products_dependencies_message_terms_title'
            ],
            'product_categories_quantity_dependency_message' => [
                'name' => esc_html__('Categories quantity dependency message', 'woocommerce-products-dependencies'),
                'default' => esc_html__(
                    'To add more of $product$ to your cart need to add more from category:',
                    'woocommerce-products-dependencies'
                ),
                'desc' => esc_html__('Items for using: $product$ - product name', 'woocommerce-products-dependencies'),
                'type' => 'text',
                'css' => 'width: 100%',
                'id' => 'woocommerce_products_dependencies_message_terms_quantity_title'
            ],
            'products_dependency_message' => [
                'name' => esc_html__('Products dependencies message', 'woocommerce-products-dependencies'),
                'default' => esc_html__('Before adding $product$ to your cart need to add:', 'woocommerce-products-dependencies'),
                'desc' =>  esc_html__('Items for using: $product$ - product name', 'woocommerce-products-dependencies'),
                'type' => 'text',
                'css' => 'width: 100%',
                'id' => 'woocommerce_products_dependencies_message_products_title'
            ],
            'products_quantity_dependency_message' => [
                'name' => esc_html__('Products quantity dependency message', 'woocommerce-products-dependencies'),
                'default' => esc_html__(
                    'To add more of $product$ to your cart need to add more of:',
                    'woocommerce-products-dependencies'
                ),
                'desc' => esc_html__('Items for using: $product$ - product name', 'woocommerce-products-dependencies'),
                'type' => 'text',
                'css' => 'width: 100%',
                'id' => 'woocommerce_products_dependencies_message_products_quantity_title'
            ],
            'products_incompatibility_message' => [
                'name' => esc_html__('Products incompatibility message', 'woocommerce-products-dependencies'),
                'default' => esc_html__('You can\'t buy $product$ with:', 'woocommerce-products-dependencies'),
                'desc' => esc_html__('Items for using: $product$ - product name', 'woocommerce-products-dependencies'),
                'type' => 'text',
                'css' => 'width: 100%',
                'id' => 'woocommerce_products_dependencies_message_products_incompatibility_title'
            ],
            'product_categories_incompatibility_message' => [
                'name' => esc_html__('Categories incompatibility message', 'woocommerce-products-dependencies'),
                'default' => esc_html__(
                    'You can\'t buy anything from this category with:',
                    'woocommerce-products-dependencies'
                ),
                'type' => 'text',
                'css' => 'width: 100%',
                'id' => 'woocommerce_products_dependencies_message_terms_incompatibility_title'
            ],
            'min_products_quantity_message' => [
                'name' => esc_html__('Minimum products quantity message', 'woocommerce-products-dependencies'),
                'desc' => esc_html__(
                    'Items for using: $n$ - products quantity; $category$ - category name',
                    'woocommerce-products-dependencies'
                ),
                'default' => esc_html__(
                    'You should buy $n$ or more products from the category "$category$"',
                    'woocommerce-products-dependencies'
                ),
                'type' => 'text',
                'css' => 'width: 100%',
                'id' => 'woocommerce_products_dependencies_message_minimum_products_quantity'
            ],
            'min_products_selected_message' => [
                'name' => esc_html__('Minimum products selected message', 'woocommerce-products-dependencies'),
                'desc' => esc_html__(
                    'Items for using: $n$ - products quantity; $category$ - category name',
                    'woocommerce-products-dependencies'
                ),
                'default' => esc_html__(
                    'You should select $n$ or more products from the category "$category$"',
                    'woocommerce-products-dependencies'
                ),
                'type' => 'text',
                'css' => 'width: 100%',
                'id' => 'woocommerce_products_dependencies_message_minimum_products_selected'
            ],
            'dependency_message_or_string' => [
                'name' => esc_html__('Dependency "OR" text', 'woocommerce-products-dependencies'),
                'default' => esc_html__('or', 'woocommerce-products-dependencies'),
                'type' => 'text',
                'id' => 'woocommerce_products_dependencies_message_or'
            ],
            'dependency_message_and_string' => [
                'name' => esc_html__('Dependency "AND" text', 'woocommerce-products-dependencies'),
                'default' => esc_html__('and', 'woocommerce-products-dependencies'),
                'type' => 'text',
                'id' => 'woocommerce_products_dependencies_message_and'
            ],
            'max_orders_per_period' => [
                'name' => esc_html__('Maximum orders per time period', 'woocommerce-products-dependencies'),
                'default' => 0,
                'type' => 'number',
                'custom_attributes' => ['min' => 0],
                'id' => 'woocommerce_products_dependencies_max_orders_per_period',
                'desc' => esc_html__('Zero is equal infinity', 'woocommerce-products-dependencies')
            ],
            'max_orders_time_period' => [
                'name' => esc_html__('Maximum orders time period in minutes', 'woocommerce-products-dependencies'),
                'default' => 1,
                'type' => 'number',
                'custom_attributes' => [
                    'min' => 0,
                    'step' => 'any'
                ],
                'id' => 'woocommerce_products_dependencies_orders_time_period'
            ],
            'max_orders_per_period_message' => [
                'name' => esc_html__('Maximum orders per time period message', 'woocommerce-products-dependencies'),
                'default' => esc_html__(
                    'Maximum orders per time period limit have achieved',
                    'woocommerce-products-dependencies'
                ),
                'type' => 'text',
                'css' => 'width: 100%',
                'id' => 'woocommerce_products_dependencies_max_orders_per_period_message'
            ],
            'prepayment_bundle_label' => [
                'name' => esc_html__('Prepayment bundle message', 'woocommerce-products-dependencies'),
                'desc' =>
                    esc_html__('Then the user has no paid items', 'woocommerce-products-dependencies')
                    . '<br>'
                    // phpcs:disable
                    . esc_html__(
                        'Items for using: $bundle$ - bundle size; $payable$ - payable products quantity; $paid$ - paid products quantity',
                        'woocommerce-products-dependencies'
                    ),
                // phpcs:enable
                'default' => esc_html__('(Prepayment for $payable$ items)', 'woocommerce-products-dependencies'),
                'type' => 'text',
                'css' => 'width: 100%',
                'id' => 'woocommerce_products_dependencies_message_prepayment_bundle'
            ],
            'paid_products_label' => [
                'name' => esc_html__('Paid products from a bundle message', 'woocommerce-products-dependencies'),
                'desc' =>
                    esc_html__(
                        'Then the user has not enough paid items and need to pay for the new bundle',
                        'woocommerce-products-dependencies'
                    )
                    . '<br>'
                    // phpcs:disable
                    . esc_html__(
                        'Items for using: $bundle$ - bundle size; $payable$ - payable products quantity; $paid$ - paid products quantity',
                        'woocommerce-products-dependencies'
                    ),
                    // phpcs:enable
                'default' => esc_html__('($paid$ items are paid already)', 'woocommerce-products-dependencies'),
                'type' => 'text',
                'css' => 'width: 100%',
                'id' => 'woocommerce_products_dependencies_message_paid_products'
            ],
            'prepayment_bundle_with_paid_products_label' => [
                'name' => esc_html__(
                    'Prepayment bundle with paid products message',
                    'woocommerce-products-dependencies'
                ),
                'desc' =>
                    esc_html__('Then the user has enough paid items', 'woocommerce-products-dependencies')
                    . '<br>'
                    // phpcs:disable
                    . esc_html__(
                        'Items for using: $bundle$ - bundle size; $payable$ - payable products quantity; $paid$ - paid products quantity',
                        'woocommerce-products-dependencies'
                    ),
                    // phpcs:enable
                'default' => esc_html__(
                    '(Prepayment for $payable$ items. $paid$ items are paid already)',
                    'woocommerce-products-dependencies'
                ),
                'type' => 'text',
                'css' => 'width: 100%',
                'id' => 'woocommerce_products_dependencies_message_prepayment_bundle_paid_products'
            ]
        ];

        $productCategory = [
            'logged_dependency' => [
                'label' => esc_html__('User must be logged to buy', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_logged_dependency',
                'default' => false,
                'type' => 'checkbox'
            ],
            'buy_once' => [
                'label' => esc_html__('Buy only once', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_buy_once',
                'default' => false,
                'type' => 'checkbox',
                'description' => esc_html__('For logged users only', 'woocommerce-products-dependencies')
            ],
            'product_categories_dependencies' => [
                'label' => esc_html__('Dependencies by categories', 'woocommerce-products-dependencies'),
                'type' => 'data-table',
                'inModal' => true,
                'key' => '_wcpd_terms_dependencies',
                'default' => [],
                'values' => [
                    [
                        'label' => esc_html__('Categories', 'woocommerce-products-dependencies'),
                        'key' => 'terms',
                        'type' => 'wc-terms-search',
                        'width' => '50%',
                        'multiple' => true,
                        'default' => []
                    ],
                    [
                        'label' => esc_html__('Min quantity', 'woocommerce-products-dependencies'),
                        'key' => 'min_quantity',
                        'type' => 'number',
                        'min' => 1,
                        'default' => 1
                    ],
                    [
                        'label' => esc_html__('Inner relation', 'woocommerce-products-dependencies'),
                        'key' => 'inner_relation',
                        'type' => 'select',
                        'default' => 'and',
                        'values' => [
                            'and' => esc_html__('AND', 'woocommerce-products-dependencies'),
                            'or' => esc_html__('OR', 'woocommerce-products-dependencies')
                        ]
                    ],
                    [
                        'label' => esc_html__('Outer relation', 'woocommerce-products-dependencies'),
                        'key' => 'outer_relation',
                        'type' => 'select',
                        'default' => 'or',
                        'values' => [
                            'and' => esc_html__('AND', 'woocommerce-products-dependencies'),
                            'or' => esc_html__('OR', 'woocommerce-products-dependencies')
                        ]
                    ]
                ]
            ],
            'quantity_dependency' => [
                'label' => esc_html__('Quantity dependency', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_quantity_dependency',
                'type' => 'checkbox',
                'default' => false,
                'description' => esc_html__(
                    'Limit maximum product quantity by master products',
                    'woocommerce-products-dependencies'
                )
            ],
            'quantity_dependency_ratio' => [
                'label' => esc_html__('Quantity dependency ratio', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_quantity_dependency_ratio',
                'type' => 'number',
                'step' => 'any',
                'default' => 1,
                'description' => esc_html__(
                    'Set dependant:master products quantity ratio: "1" = 1:1, "2" = 1:2, "0.5" = 2:1 and so on',
                    'woocommerce-products-dependencies'
                )
            ],
            'incompatible_product_categories' => [
                'label' => esc_html__('Incompatible categories', 'woocommerce-products-dependencies'),
                'type' => 'wc-terms-search',
                'key' => '_wcpd_incompatible_terms',
                'multiple' => true,
                'default' => [],
                'description' =>
                    esc_html__(
                        'Can\'t buy from this category then incompatible category products are in the cart',
                        'woocommerce-products-dependencies'
                    )
                    . '<br>'
                    . esc_html__(
                        'Add this category as incompatible to selected categories too to make it work perfectly',
                        'woocommerce-products-dependencies'
                    )
            ],
            'min_product_quantity' => [
                'label' => esc_html__('Minimum product quantity per order', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_min_product_quantity',
                'type' => 'group',
                'showHeader' => true,
                'default' => [],
                'description' => esc_html__(
                    'Limit minimum quantity input value',
                    'woocommerce-products-dependencies'
                ),
                'values' => [
                    [
                        'label' => esc_html__('Type', 'woocommerce-products-dependencies'),
                        'key' => 'type',
                        'type' => 'select',
                        'default' => 'count',
                        'values' => [
                            'count' => esc_html__('Simple count', 'woocommerce-products-dependencies'),
                            'terms-least' => esc_html__(
                                'Least of product count from categories',
                                'woocommerce-products-dependencies'
                            ),
                            'terms-greatest' => esc_html__(
                                'Greatest of product count from categories',
                                'woocommerce-products-dependencies'
                            ),
                            'terms-sum' =>
                                esc_html__('Sum of products count from categories', 'woocommerce-products-dependencies')
                        ]
                    ],
                    [
                        'label' => esc_html__('Count', 'woocommerce-products-dependencies'),
                        'key' => 'count',
                        'type' => 'number',
                        'min' => 0,
                        'default' => 0
                    ],
                    [
                        'label' => esc_html__('Categories', 'woocommerce-products-dependencies'),
                        'key' => 'terms',
                        'type' => 'wc-terms-search',
                        'width' => '50%',
                        'multiple' => true,
                        'default' => []
                    ]
                ]
            ],
            'min_products_quantity' => [
                'label' => esc_html__(
                    'Minimum total products quantity per order',
                    'woocommerce-products-dependencies'
                ),
                'key' => '_wcpd_min_products_quantity',
                'type' => 'group',
                'showHeader' => true,
                'default' => [],
                'description' => esc_html__(
                    'Limit minimum total products quantity of the category',
                    'woocommerce-products-dependencies'
                ),
                'values' => [
                    [
                        'label' => esc_html__('Type', 'woocommerce-products-dependencies'),
                        'key' => 'type',
                        'type' => 'select',
                        'default' => 'count',
                        'values' => [
                            'count' => esc_html__('Simple count', 'woocommerce-products-dependencies'),
                            'terms-least' => esc_html__(
                                'Least of product count from categories',
                                'woocommerce-products-dependencies'
                            ),
                            'terms-greatest' => esc_html__(
                                'Greatest of product count from categories',
                                'woocommerce-products-dependencies'
                            ),
                            'terms-sum' =>
                                esc_html__('Sum of products count from categories', 'woocommerce-products-dependencies')
                        ]
                    ],
                    [
                        'label' => esc_html__('Count', 'woocommerce-products-dependencies'),
                        'key' => 'count',
                        'type' => 'number',
                        'min' => 0,
                        'default' => 0
                    ],
                    [
                        'label' => esc_html__('Categories', 'woocommerce-products-dependencies'),
                        'key' => 'terms',
                        'type' => 'wc-terms-search',
                        'width' => '50%',
                        'multiple' => true,
                        'default' => []
                    ]
                ]
            ],
            'min_products_selected' => [
                'label' => esc_html__('Minimum products selected per order', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_min_products_selected',
                'type' => 'group',
                'showHeader' => true,
                'default' => [],
                'description' => esc_html__(
                    'Limit minimum products selected of the category',
                    'woocommerce-products-dependencies'
                ),
                'values' => [
                    [
                        'label' => esc_html__('Type', 'woocommerce-products-dependencies'),
                        'key' => 'type',
                        'type' => 'select',
                        'default' => 'count',
                        'values' => [
                            'count' => esc_html__('Simple count', 'woocommerce-products-dependencies'),
                            'terms-count' =>
                                esc_html__('Categories selected products count', 'woocommerce-products-dependencies')
                        ]
                    ],
                    [
                        'label' => esc_html__('Count', 'woocommerce-products-dependencies'),
                        'key' => 'count',
                        'type' => 'number',
                        'min' => 0,
                        'default' => 0
                    ],
                    [
                        'label' => esc_html__('Categories', 'woocommerce-products-dependencies'),
                        'key' => 'terms',
                        'type' => 'wc-terms-search',
                        'width' => '50%',
                        'multiple' => true,
                        'default' => []
                    ]
                ]
            ],
            'prepayment_bundle_size' => [
                'label' => esc_html__('Prepayment bundle size', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_prepayment_bundle_size',
                'type' => 'number',
                'min' => 0,
                'default' => 0,
                // phpcs:disable
                'description' => esc_html__(
                    'Make products be paid for a bundle. Paid but not ordered yet products will be kept for the customer for further orders as free. Works for logged customers only!',
                    'woocommerce-products-dependencies'
                )
                // phpcs:enable
            ],
            'prepayment_bundle_regularity' => [
                'label' => esc_html__('Prepayment bundle regularity', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_prepayment_bundle_regularity',
                'type' => 'select',
                'default' => 'first-order',
                'description' => esc_html__(
                    'Make bundles regular or only for the first time order',
                    'woocommerce-products-dependencies'
                ),
                'values' => [
                    'first-order' => esc_html__('First order', 'woocommerce-products-dependencies'),
                    'each-order' => esc_html__('Each order', 'woocommerce-products-dependencies')
                ]
            ],
            'enable_custom_dependencies_message' => [
                'label' =>
                    esc_html__('Overwrite default dependencies message by the custom', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_enable_custom_message',
                'default' => false,
                'type' => 'checkbox'
            ],
            'custom_dependencies_message' => [
                'label' => esc_html__('Custom message', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_custom_message',
                'default' => '',
                'type' => 'textarea'
            ]
        ];

        $product = [
            'products_dependencies' => [
                'label' => esc_html__('Dependencies by products', 'woocommerce-products-dependencies'),
                'type' => 'data-table',
                'inModal' => true,
                'key' => '_wcpd_products_dependencies',
                'default' => [],
                'values' => [
                    [
                        'label' => esc_html__('Products', 'woocommerce-products-dependencies'),
                        'key' => 'products',
                        'type' => 'wc-product-search',
                        'width' => '50%',
                        // 'action' => 'woocommerce_json_search_products',
                        'multiple' => true,
                        'default' => []
                    ],
                    [
                        'label' => esc_html__('Min quantity', 'woocommerce-products-dependencies'),
                        'key' => 'min_quantity',
                        'type' => 'number',
                        'min' => 1,
                        'default' => 1
                    ],
                    [
                        'label' => esc_html__('Inner relation', 'woocommerce-products-dependencies'),
                        'key' => 'inner_relation',
                        'type' => 'select',
                        'default' => 'and',
                        'values' => [
                            'and' => esc_html__('AND', 'woocommerce-products-dependencies'),
                            'or' => esc_html__('OR', 'woocommerce-products-dependencies')
                        ]
                    ],
                    [
                        'label' => esc_html__('Outer relation', 'woocommerce-products-dependencies'),
                        'key' => 'outer_relation',
                        'type' => 'select',
                        'default' => 'or',
                        'values' => [
                            'and' => esc_html__('AND', 'woocommerce-products-dependencies'),
                            'or' => esc_html__('OR', 'woocommerce-products-dependencies')
                        ]
                    ]
                ]
            ],
            'product_categories_dependencies' => [
                'label' => esc_html__('Dependencies by categories', 'woocommerce-products-dependencies'),
                'type' => 'data-table',
                'inModal' => true,
                'key' => '_wcpd_terms_dependencies',
                'default' => [],
                'values' => [
                    [
                        'label' => esc_html__('Categories', 'woocommerce-products-dependencies'),
                        'key' => 'terms',
                        'type' => 'wc-terms-search',
                        'width' => '50%',
                        'multiple' => true,
                        'default' => []
                    ],
                    [
                        'label' => esc_html__('Min quantity', 'woocommerce-products-dependencies'),
                        'key' => 'min_quantity',
                        'type' => 'number',
                        'min' => 1,
                        'default' => 1
                    ],
                    [
                        'label' => esc_html__('Inner relation', 'woocommerce-products-dependencies'),
                        'key' => 'inner_relation',
                        'type' => 'select',
                        'default' => 'and',
                        'values' => [
                            'and' => esc_html__('AND', 'woocommerce-products-dependencies'),
                            'or' => esc_html__('OR', 'woocommerce-products-dependencies')
                        ]
                    ],
                    [
                        'label' => esc_html__('Outer relation', 'woocommerce-products-dependencies'),
                        'key' => 'outer_relation',
                        'type' => 'select',
                        'default' => 'or',
                        'values' => [
                            'and' => esc_html__('AND', 'woocommerce-products-dependencies'),
                            'or' => esc_html__('OR', 'woocommerce-products-dependencies')
                        ]
                    ]
                ]
            ],
            'quantity_dependency' => [
                'label' => esc_html__('Quantity dependency', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_quantity_dependency',
                'type' => 'checkbox',
                'default' => false,
                'description' => esc_html__(
                    'Limit maximum product quantity by master products',
                    'woocommerce-products-dependencies'
                )
            ],
            'quantity_dependency_ratio' => [
                'label' => esc_html__('Quantity dependency ratio', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_quantity_dependency_ratio',
                'type' => 'number',
                'step' => 'any',
                'default' => 1,
                'description' => esc_html__(
                    'Set dependant:master products quantity ratio: "1" = 1:1, "2" = 1:2, "0.5" = 2:1 and so on',
                    'woocommerce-products-dependencies'
                )
            ],
            'incompatible_products' => [
                'label' => esc_html__('Incompatible products', 'woocommerce-products-dependencies'),
                'type' => 'wc-product-search',
                'key' => '_wcpd_incompatible_products',
                'multiple' => true,
                'default' => [],
                'description' =>
                    esc_html__(
                        'Can\'t buy this product then incompatible products are in the cart',
                        'woocommerce-products-dependencies'
                    )
                    . '<br>'
                    . esc_html__(
                        'Add this product as incompatible to selected products too to make it work perfectly',
                        'woocommerce-products-dependencies'
                    )
            ],
            'logged_dependency' => [
                'label' => esc_html__('User must be logged to buy', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_logged_dependency',
                'default' => false,
                'type' => 'checkbox'
            ],
            'buy_once' => [
                'label' => esc_html__('Buy only once', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_buy_once',
                'default' => false,
                'type' => 'checkbox',
                'description' => esc_html__('For logged users only', 'woocommerce-products-dependencies')
            ],
            'visibility_by_users' => [
                'label' => esc_html__('Visibility by users', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_visibility_by_users',
                'default' => [],
                'type' => 'wc-user-search',
                'values' => $usersData,
                'description' => esc_html__(
                    'Make product visible only to selected logged users',
                    'woocommerce-products-dependencies'
                )
            ],
            'enable_individual_min_quantity' => [
                'label' => esc_html__(
                    'Overwrite category minimum product quantity rule',
                    'woocommerce-products-dependencies'
                ),
                'key' => '_wcpd_enable_individual_min_quantity',
                'default' => false,
                'type' => 'checkbox'
            ],
            'min_quantity' => [
                'label' => esc_html__('Minimum product quantity per order', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_min_quantity',
                'type' => 'group',
                'showHeader' => true,
                'description' => esc_html__(
                    'Limit minimum quantity input value',
                    'woocommerce-products-dependencies'
                ),
                'default' => [],
                'values' => [
                    [
                        'label' => esc_html__('Type', 'woocommerce-products-dependencies'),
                        'key' => 'type',
                        'type' => 'select',
                        'default' => 'count',
                        'values' => [
                            'count' => esc_html__('Simple count', 'woocommerce-products-dependencies'),
                            'terms-least' => esc_html__(
                                'Least of product count from categories',
                                'woocommerce-products-dependencies'
                            ),
                            'terms-greatest' => esc_html__(
                                'Greatest of product count from categories',
                                'woocommerce-products-dependencies'
                            ),
                            'terms-sum' => esc_html__(
                                'Sum of products count from categories',
                                'woocommerce-products-dependencies'
                            )
                        ]
                    ],
                    [
                        'label' => esc_html__('Count', 'woocommerce-products-dependencies'),
                        'key' => 'count',
                        'type' => 'number',
                        'min' => 0,
                        'default' => 0
                    ],
                    [
                        'label' => esc_html__('Categories', 'woocommerce-products-dependencies'),
                        'key' => 'terms',
                        'type' => 'wc-terms-search',
                        'width' => '50%',
                        'multiple' => true,
                        'default' => []
                    ]
                ]
            ],
            'enable_custom_dependencies_message' => [
                'label' =>
                    esc_html__('Overwrite default dependencies message by the custom', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_enable_custom_message',
                'default' => false,
                'type' => 'checkbox'
            ],
            'custom_dependencies_message' => [
                'label' => esc_html__('Custom message', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_custom_message',
                'default' => '',
                'type' => 'textarea'
            ]
        ];

        $productVariation = [
            'products_dependencies' => [
                'label' => esc_html__('Dependencies by products', 'woocommerce-products-dependencies'),
                'type' => 'data-table',
                'inModal' => true,
                'key' => '_wcpd_variations_dependencies',
                'default' => [],
                'values' => [
                    [
                        'label' => esc_html__('Products', 'woocommerce-products-dependencies'),
                        'key' => 'products',
                        'type' => 'wc-product-search',
                        'width' => '50%',
                        // 'action' => 'woocommerce_json_search_products',
                        'multiple' => true,
                        'default' => []
                    ],
                    [
                        'label' => esc_html__('Min quantity', 'woocommerce-products-dependencies'),
                        'key' => 'min_quantity',
                        'type' => 'number',
                        'min' => 1,
                        'default' => 1
                    ],
                    [
                        'label' => esc_html__('Inner relation', 'woocommerce-products-dependencies'),
                        'key' => 'inner_relation',
                        'type' => 'select',
                        'default' => 'and',
                        'values' => [
                            'and' => esc_html__('AND', 'woocommerce-products-dependencies'),
                            'or' => esc_html__('OR', 'woocommerce-products-dependencies')
                        ]
                    ],
                    [
                        'label' => esc_html__('Outer relation', 'woocommerce-products-dependencies'),
                        'key' => 'outer_relation',
                        'type' => 'select',
                        'default' => 'or',
                        'values' => [
                            'and' => esc_html__('AND', 'woocommerce-products-dependencies'),
                            'or' => esc_html__('OR', 'woocommerce-products-dependencies')
                        ]
                    ]
                ]
            ],
            'product_categories_dependencies' => [
                'label' => esc_html__('Dependencies by categories', 'woocommerce-products-dependencies'),
                'type' => 'data-table',
                'inModal' => true,
                'key' => '_wcpd_variation_terms_dependencies',
                'default' => [],
                'values' => [
                    [
                        'label' => esc_html__('Categories', 'woocommerce-products-dependencies'),
                        'key' => 'terms',
                        'type' => 'wc-terms-search',
                        'width' => '50%',
                        'multiple' => true,
                        'default' => []
                    ],
                    [
                        'label' => esc_html__('Min quantity', 'woocommerce-products-dependencies'),
                        'key' => 'min_quantity',
                        'type' => 'number',
                        'min' => 1,
                        'default' => 1
                    ],
                    [
                        'label' => esc_html__('Inner relation', 'woocommerce-products-dependencies'),
                        'key' => 'inner_relation',
                        'type' => 'select',
                        'default' => 'and',
                        'values' => [
                            'and' => esc_html__('AND', 'woocommerce-products-dependencies'),
                            'or' => esc_html__('OR', 'woocommerce-products-dependencies')
                        ]
                    ],
                    [
                        'label' => esc_html__('Outer relation', 'woocommerce-products-dependencies'),
                        'key' => 'outer_relation',
                        'type' => 'select',
                        'default' => 'or',
                        'values' => [
                            'and' => esc_html__('AND', 'woocommerce-products-dependencies'),
                            'or' => esc_html__('OR', 'woocommerce-products-dependencies')
                        ]
                    ]
                ]
            ],
            'quantity_dependency' => [
                'label' => esc_html__('Quantity dependency', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_variation_quantity_dependency',
                'type' => 'checkbox',
                'default' => false,
                'description' => esc_html__(
                    'Limit maximum product quantity by master products',
                    'woocommerce-products-dependencies'
                )
            ],
            'quantity_dependency_ratio' => [
                'label' => esc_html__('Quantity dependency ratio', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_variation_quantity_dependency_ratio',
                'type' => 'number',
                'step' => 'any',
                'default' => 1,
                'description' => esc_html__(
                    'Set dependant:master products quantity ratio: "1" = 1:1, "2" = 1:2, "0.5" = 2:1 and so on',
                    'woocommerce-products-dependencies'
                )
            ],
            'incompatible_products' => [
                'label' => esc_html__('Incompatible products', 'woocommerce-products-dependencies'),
                'type' => 'wc-product-search',
                'key' => '_wcpd_variation_incompatible_products',
                'multiple' => true,
                'default' => [],
                'description' =>
                    esc_html__(
                        'Can\'t buy this variation then incompatible products are in the cart',
                        'woocommerce-products-dependencies'
                    )
                    . '<br>'
                    . esc_html__(
                        'Add this variation as incompatible to selected products too to make it work perfectly',
                        'woocommerce-products-dependencies'
                    )
            ],
            'logged_dependency' => [
                'label' => esc_html__('User must be logged to buy', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_variation_logged_dependency',
                'default' => false,
                'type' => 'checkbox'
            ],
            'enable_custom_dependencies_message' => [
                'label' =>
                    esc_html__('Overwrite default message by the custom', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_variation_enable_custom_message',
                'default' => false,
                'type' => 'checkbox'
            ],
            'custom_dependencies_message' => [
                'label' => esc_html__('Custom message', 'woocommerce-products-dependencies'),
                'key' => '_wcpd_variation_custom_message',
                'default' => '',
                'type' => 'textarea'
            ]
        ];

        $modelCache = [
            'global' => apply_filters('wcpd_global_settings_model', $global),
            'product' => apply_filters('wcpd_product_settings_model', $product),
            'productVariation' => apply_filters('wcpd_product_variation_settings_model', $productVariation),
            'productCategory' => apply_filters('wcpd_product_category_settings_model', $productCategory)
        ];

        if ($source && isset($modelCache[$source])) {
            return apply_filters('wcpd_settings_model_' . $source, $modelCache[$source]);
        }

        return apply_filters('wcpd_settings_models', $modelCache);
    }

    /**
     * Get one of the global settings
     *
     * @param string $setting
     *
     * @return string|boolean|array
     */
    public static function getGlobal($setting)
    {
        $model = self::getModel('global');

        if (!isset($model[$setting])) {
            return null;
        }

        static $globalSettingsCache;

        if (isset($globalSettingsCache[$setting])) {
            $value = $globalSettingsCache[$setting];

            return apply_filters('wcpd_global_setting', $value, $setting);
        }

        $value = isset($model[$setting]['id']) ? get_option($model[$setting]['id'], null) : null;

        if ($value == null && isset($model[$setting]['default'])) {
            $value = $model[$setting]['default'];
        }

        $value = self::handleSettingType($value, $model[$setting]['type']);
        $globalSettingsCache[$setting] = $value;

        return apply_filters('wcpd_global_setting', $value, $setting);
    }

    /**
     * Get one of post settings
     *
     * @param integer $id
     * @param string $setting
     * @param string $modelSource
     * @param mixed $default
     *
     * @return string|float|boolean|array
     */
    public static function getPost($id, $setting, $modelSource = 'post', $default = null)
    {
        $model = self::getModel($modelSource);

        static $postSettingsCache;

        if (isset($postSettingsCache[$id], $postSettingsCache[$id][$setting])) {
            $value = $postSettingsCache[$id][$setting];

            return apply_filters('wcpd_post_setting', $value, $id, $setting, $modelSource);
        }

        if (!isset($model[$setting])) {
            $postSettingsCache[$id][$setting] = $default;

            return apply_filters('wcpd_post_setting', $default, $id, $setting, $modelSource);
        }

        $value = isset($model[$setting]['key']) ? get_post_meta($id, $model[$setting]['key'], true) : null;

        if ($value == null) {
            if ($default) {
                $value = $default;
            } elseif (isset($model[$setting]['default'])) {
            $value = $model[$setting]['default'];
        }
        }

        $value = self::handleSettingType($value, $model[$setting]['type']);

        if (!isset($postSettingsCache[$id])) {
            $postSettingsCache[$id] = [];
        }

        $postSettingsCache[$id][$setting] = $value;

        return apply_filters('wcpd_post_setting', $value, $id, $setting, $modelSource);
    }

    /**
     * Get product setting
     *
     * @param integer $id
     * @param string $setting
     * @param mixed $default
     *
     * @return string|float|bool|array
     */
    public static function getProduct($id, $setting, $default = null)
    {
        return self::getPost($id, $setting, 'product', $default);
    }

    /**
     * Get product variation setting
     *
     * @param integer $id
     * @param string $setting
     * @param mixed $default
     *
     * @return string|float|bool|array
     */
    public static function getProductVariation($id, $setting, $default = null)
    {
        return self::getPost($id, $setting, 'productVariation', $default);
    }

    /**
     * Get product category setting
     *
     * @param integer $id
     * @param string $setting
     * @param mixed $default
     *
     * @return string|float|bool|array
     */
    public static function getProductCategory($id, $setting, $default = null)
    {
        $model = self::getModel('productCategory');

        static $productCategorySettingsCache;

        if (isset($productCategorySettingsCache[$id], $productCategorySettingsCache[$id][$setting])) {
            $value = $productCategorySettingsCache[$id][$setting];

            return apply_filters('wcpd_product_category_setting', $value, $id, $setting);
        }

        if (!isset($model[$setting])) {
            $productCategorySettingsCache[$id][$setting] = $default;

            return apply_filters('wcpd_product_category_setting', $default, $id, $setting);
        }

        $value = isset($model[$setting]['key']) ? get_term_meta($id, $model[$setting]['key'], true) : null;

        if ($value == null) {
            if ($default) {
                $value = $default;
            } elseif (isset($model[$setting]['default'])) {
            $value = $model[$setting]['default'];
        }
        }

        $value = self::handleSettingType($value, $model[$setting]['type']);

        if (!isset($productCategorySettingsCache[$id])) {
            $productCategorySettingsCache[$id] = [];
        }

        $productCategorySettingsCache[$id][$setting] = $value;

        return apply_filters('wcpd_product_category_setting', $value, $id, $setting);
    }

    /**
     * Handle the setting value according to the type
     *
     * @param mixed $value
     * @param string $type
     *
     * @return string|float|bool|array
     */
    public static function handleSettingType($value, $type = 'string')
    {
        switch ($type) {
            case 'checkbox':
                return filter_var($value, FILTER_VALIDATE_BOOLEAN);

            case 'number':
                return (float) $value;

            case 'array':
                return (array) $value;

            case 'string':
                return (string) $value;
        }

        return $value;
    }
}
