/* WooCommerce Products Dependencies Data Table
 * Original author: troll_winner@mail.ru
 * Further changes, comments: troll_winner@mail.ru
 */

(function (root, factory) {
    'use strict';

    if (typeof define === 'function' && define.amd) {
        define(['jquery'], factory);
    } else if (typeof exports === 'object'
        && typeof module !== 'undefined'
        && typeof require === 'function'
    ) {
        module.exports = factory(require('jquery'));
    } else {
        factory(root.jQuery);
    }
})(this, function ($) {
    'use strict';

    const pluginName = 'wcpdDataTable';
    const defaults = {};
    const $document = $(document);

    const Plugin = function (element, options) {
        this.element = element;
        this.options = $.extend({}, defaults, options);

        return this.init();
    };

    // on plugin init
    Plugin.prototype.init = function () {
        this.$element = $(this.element);
        this.$template = this.$element.find('[data-component~="wcpd-data-table-item-template"]');

        if ($.fn.sortable) {
            // init sortable
            this.$element.sortable({
                items: '[data-component~="wcpd-data-table-item"]',
                update: () => this.recalculation(),
                cancel: 'input,textarea,button,select,option,.wcpd-modal'
            });
        }

        return this;
    };

    // add a new element in the table
    Plugin.prototype.addItem = function ($insertAfterItem, $template = this.$template) {
        $insertAfterItem = $insertAfterItem.length !== 0
            ? $insertAfterItem
            : this.$element.find('[data-component~="wcpd-data-table-item"]:last');

        if ($template.length === 0) {
            $template = $insertAfterItem;
        }

        // clone the new element from default template
        const $clone = $template.clone();

        $clone
            .attr('data-component', 'wcpd-data-table-item')
            .find(':input')
            .each(function () {
                const $this = $(this);

                // make real attributes from the placeholders
                $.each(this.attributes, (i, attr) => {
                    const name = attr.name;
                    const value = attr.value;

                    if (name.indexOf('data-make') === -1) {
                        return this;
                    }

                    return $this.attr(name.replace('data-make-', ''), value);
                });
            });

        const $settingsModal = $clone.find('[data-component~="wcpd-data-table-modal"]');
        const $openSettingsModal = $clone.find('[data-component~="wcpd-data-table-open-modal"]');
        const rand = Math.random().toString(36).substr(2);

        $settingsModal.attr('id', $settingsModal.attr('id') + `-${rand.toString()}`);
        $openSettingsModal.attr('href', $openSettingsModal.attr('href') + `-${rand.toString()}`);

        // insert the clone element
        $clone.insertAfter($insertAfterItem);

        this.recalculation();

        $document.trigger('added.item.dataTable.wcpd', [this, $clone]);

        return this;
    };

    // remove element from the table
    Plugin.prototype.removeItem = function ($item) {
        if ($item.is(':only-child')) {
            // clear values of the last item
            $item.find(':input').val('').trigger('change');
        } else {
            // remove the non-last item
            $item.remove();
        }

        this.recalculation();

        $document.trigger('removed.item.dataTable.wcpd', [this]);

        return this;
    };

    // clone element from the table
    Plugin.prototype.cloneItem = function ($item) {
        this.addItem($item, $item);

        $document.trigger('cloned.item.dataTable.wcpd', [this]);

        return this;
    };

    // reCalculate the input names indexes
    Plugin.prototype.recalculation = function () {
        return this.$element
            .find('[data-component~="wcpd-data-table-item"]')
            .each(function (index) {
                return $(this)
                    .find(':input')
                    .each(function () {
                        const $input = $(this);
                        const name = $input.attr('name');

                        if (!name) {
                            return this;
                        }

                        const number = index.toString().split('').reverse().join('');

                        // replace the first array key from the end
                        return $input.attr(
                            'name',
                            name
                                .split('')
                                .reverse()
                                .join('')
                                .replace(/]\d+\[/, `]${number}[`)
                                .split('')
                                .reverse()
                                .join('')
                        );
                    });
            });
    };

    $.fn[pluginName] = function (options) {
        return this.each(function () {
            if (!$.data(this, 'wcpd-data-table')) {
                $.data(this, 'wcpd-data-table', new Plugin(this, options));
            }
        });
    };

    const init = () => $('[data-component~="wcpd-data-table"]').each(function () {
        return $(this).wcpdDataTable();
    });

    $document
        .ready(() => init())
        .on('ajaxComplete.wcpd init.dataTable.wcpd', () => init())

        // add the form item
        .on('click', '[data-component~="wcpd-data-table-item-add"]', function () {
            const $button = $(this);
            const $dataTable = $button.closest('[data-component~="wcpd-data-table"]');

            if ($dataTable.data('wcpd-data-table')) {
                return $dataTable
                    .data('wcpd-data-table')
                    .addItem($button.closest('[data-component~="wcpd-data-table-item"]'));
            }

            return this;
        })

        // remove the form item
        .on('click', '[data-component~="wcpd-data-table-item-remove"]', function () {
            const $button = $(this);
            const $dataTable = $button.closest('[data-component~="wcpd-data-table"]');

            if ($dataTable.data('wcpd-data-table')) {
                return $dataTable
                    .data('wcpd-data-table')
                    .removeItem($button.closest('[data-component~="wcpd-data-table-item"]'));
            }

            return this;
        })

        // clone the form item
        .on('click', '[data-component~="wcpd-data-table-item-clone"]', function () {
            const $button = $(this);
            const $dataTable = $button.closest('[data-component~="wcpd-data-table"]');

            if ($dataTable.data('wcpd-data-table')) {
                return $dataTable
                    .data('wcpd-data-table')
                    .cloneItem($button.closest('[data-component~="wcpd-data-table-item"]'));
            }

            return this;
        });
});
